library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ASB004_07 is
	Port ( CLK18M: in  std_logic; -- SYSTEM CLOCK
			 DSW0  : in std_logic;
			 L     : out std_logic_vector(1 downto 0); -- self reset

			 RXF_N : in  std_logic;
			 TXE_N : in  std_logic;

			 RD_N  : out std_logic;
			 WR    : out std_logic;

			 USB_DATA : inout std_logic_vector( 7 downto 0 );

			 -----------------------------------------------
			 EX_RST : out std_logic;
			 HD  : inout std_logic_vector(15 downto 0);
			 HA  : out   std_logic_vector( 2 downto 0);
			 HCS : out   std_logic;
			 HWR : out   std_logic_vector( 1 downto 0);
			 HRD : out   std_logic_vector( 1 downto 0) );
end ASB004_07;

architecture RTL of ASB004_07 is

component USB_CTRL is
    Port ( CLK      : in std_logic;
	 		  RST      : in std_logic;

			  RXF_N    : in std_logic;
			  TXE_N    : in std_logic;
			  RD_N     : out std_logic;
			  WR       : out std_logic;

			  USB_DATA : inout std_logic_vector( 7 downto 0 );

			  TGT_RDY  : in  std_logic;
			  USB_BSY  : out std_logic;

			  IBIT32   : in  std_logic_vector(31 downto 0);
			  OBIT32   : out std_logic_vector(31 downto 0); 
			  IBIT_ENB : in  std_logic;
			  OBIT_ENB : out std_logic );
end component;

component EPNCTL is
    Port ( CLK : in  std_logic;
	 		  RST : in  std_logic;
			  L   : out std_logic_vector(1 downto 0);

	 		  DATA_IN  : in  std_logic_vector(31 downto 0);
           RE       : in  std_logic;
			  RE_BSY   : out std_logic;

			  DATA_OUT : out std_logic_vector(31 downto 0);
			  WE       : out std_logic;
			  WE_RDY   : in  std_logic;

			  HRST : out   std_logic; 
           HD   : inout std_logic_vector(15 downto 0);
           HA   : out   std_logic_vector(2 downto 0);
           HCS  : out   std_logic;
           HWR  : out   std_logic_vector(1 downto 0);
           HRD  : out   std_logic_vector(1 downto 0) );
end component;

signal RST   : std_logic;
signal USBDATA  : std_logic_vector(31 downto 0);
signal EPNDATA  : std_logic_vector(31 downto 0);
signal RE   : std_logic;
signal WE   : std_logic;

signal EPN_BSY : std_logic;
signal EPN_RDY : std_logic;
signal USB_BSY : std_logic;
signal USB_RDY : std_logic;

begin

RST <= not DSW0;

EPN_RDY <= not EPN_BSY;
USB_RDY <= not USB_BSY;

U0 : USB_CTRL port map
    ( CLK      => CLK18M,
		RST      => RST,
		
		RXF_N    => RXF_N,
		TXE_N    => TXE_N,
		RD_N     => RD_N,
		WR       => WR,
		
		USB_DATA => USB_DATA,
		
		TGT_RDY  => EPN_RDY,
		USB_BSY  => USB_BSY,
		
		OBIT32   => USBDATA,
		IBIT32   => EPNDATA,

 		OBIT_ENB => RE,
		IBIT_ENB => WE );

U1 : EPNCTL port map
	( CLK      => CLK18M,
	  RST      => RST,
	  L        => L,
	  DATA_IN  => USBDATA,
     RE       => RE,
	  RE_BSY   => EPN_BSY,
	  DATA_OUT => EPNDATA,
	  WE       => WE,
	  WE_RDY   => USB_RDY,
	  HRST     => EX_RST,
     HD       => HD,
     HA       => HA,
     HCS      => HCS,
     HWR      => HWR,
     HRD      => HRD );

end RTL;



