--***            *--
--**  USB_CTRL  **--
--*            ***--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity USB_CTRL is
    Port ( CLK      : in std_logic;
	 		  RST      : in std_logic;

			  RXF_N    : in std_logic;
			  TXE_N    : in std_logic;
			  RD_N     : out std_logic;
			  WR       : out std_logic;

			  USB_DATA : inout std_logic_vector( 7 downto 0 );

			  TGT_RDY  : in  std_logic;
			  USB_BSY  : out std_logic;

			  IBIT32   : in  std_logic_vector(31 downto 0);
			  OBIT32   : out std_logic_vector(31 downto 0); 

			  IBIT_ENB : in  std_logic;
			  OBIT_ENB : out std_logic );
end USB_CTRL;

architecture RTL of USB_CTRL is

component USB_ARBITER is
	Port ( CLK : in  std_logic;
			 RST : in  std_logic;

			 RXF_N : in std_logic;
			 TXE_N : in std_logic;
			 RD_N  : out std_logic;
			 WR    : out std_logic;

			 BUS_BUSY : out std_logic;
			 TG_READY : in  std_logic;

			 USB_DATA : inout  std_logic_vector( 7 downto 0 );

			 TG_DO    : out std_logic_vector( 7 downto 0 );
			 TG_DI    : in  std_logic_vector( 7 downto 0 );
			 TG_RD    : out std_logic;
			 TG_WR    : in  std_logic );
end component;

component pack8to32 is
    Port ( CLK     : in  std_logic;
	 		  RST     : in  std_logic;

			  IBIT8   : in  std_logic_vector( 7 downto 0);
			  USB_RD  : in  std_logic;

			  OBIT8   : out std_logic_vector( 7 downto 0);
			  USB_WR  : out std_logic;

			  IBIT32  : in  std_logic_vector(31 downto 0); 
			  TGT_RD  : in  std_logic;

			  OBIT32  : out std_logic_vector(31 downto 0);
			  TGT_WR  : out std_logic;

			  USB_RDY : in  std_logic;
			  RD_BSY  : out std_logic;

			  TGT_RDY : in  std_logic;
			  WR_BSY  : out std_logic );
end component;


signal USB_ARBITER_BSY : std_logic;
signal USB_ARBITER_RDY : std_logic;

signal PCK_BSY : std_logic;
signal PCK_RDY : std_logic;

signal FROM_USB : std_logic_vector( 7 downto 0 );
signal TO_USB   : std_logic_vector( 7 downto 0 );

signal FROM_USB_ENB : std_logic;
signal TO_USB_ENB   : std_logic;

begin

USB_ARBITER_RDY <= not USB_ARBITER_BSY;
PCK_RDY         <= not PCK_BSY;

U0 : USB_ARBITER port map
     ( CLK => CLK,
		 RST => RST,
		
		 RXF_N => RXF_N,
		 TXE_N => TXE_N,  
		 RD_N  => RD_N,  
		 WR    => WR,
		
		 BUS_BUSY => USB_ARBITER_BSY,
		 TG_READY => PCK_RDY,
		
		 USB_DATA => USB_DATA,
		
		 TG_DO => FROM_USB,
		 TG_DI => TO_USB,

		 TG_RD => FROM_USB_ENB,
		 TG_WR => TO_USB_ENB );
	
U1 : pack8to32 port map
    ( CLK     => CLK,
		RST     => RST,
		
		IBIT8   => FROM_USB,
		USB_RD  => FROM_USB_ENB,
		
		OBIT8   => TO_USB,
		USB_WR  => TO_USB_ENB,
		
		IBIT32  => IBIT32, 
		TGT_RD  => IBIT_ENB,
		
		OBIT32  => OBIT32,
		TGT_WR  => OBIT_ENB,
		
		USB_RDY => USB_ARBITER_RDY,
		RD_BSY  => PCK_BSY,
		
		TGT_RDY => TGT_RDY,
		WR_BSY  => USB_BSY );
		
end RTL;
