library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity pack8to32 is
    Port ( CLK     : in  std_logic;
	 		  RST     : in  std_logic;

			  IBIT8   : in  std_logic_vector( 7 downto 0);
			  USB_RD  : in  std_logic;

			  OBIT8   : out std_logic_vector( 7 downto 0);
			  USB_WR  : out std_logic;

			  IBIT32  : in  std_logic_vector(31 downto 0); 
			  TGT_RD  : in  std_logic;

			  OBIT32  : out std_logic_vector(31 downto 0);
			  TGT_WR  : out std_logic;

			  USB_RDY : in  std_logic;
			  RD_BSY  : out std_logic;

			  TGT_RDY : in  std_logic;
			  WR_BSY  : out std_logic );
end pack8to32;

architecture RTL of pack8to32 is

signal RD_STATE : std_logic_vector( 2 downto 0 );
signal WR_STATE : std_logic_vector( 3 downto 0 );

signal BIT32 : std_logic_vector(31 downto 0);

constant RS0 : std_logic_vector(2 downto 0) := "000";
constant RS1 : std_logic_vector(2 downto 0) := "001";
constant RS2 : std_logic_vector(2 downto 0) := "010";
constant RS3 : std_logic_vector(2 downto 0) := "011";
constant RS4 : std_logic_vector(2 downto 0) := "100";
constant RS5 : std_logic_vector(2 downto 0) := "101";
constant RS6 : std_logic_vector(2 downto 0) := "110";
constant RS7 : std_logic_vector(2 downto 0) := "111";

constant WS0 : std_logic_vector(3 downto 0) := "0000";
constant WS1 : std_logic_vector(3 downto 0) := "0001";
constant WS2 : std_logic_vector(3 downto 0) := "0010";
constant WS3 : std_logic_vector(3 downto 0) := "0011";
constant WS4 : std_logic_vector(3 downto 0) := "0100";
constant WS5 : std_logic_vector(3 downto 0) := "0101";
constant WS6 : std_logic_vector(3 downto 0) := "0110";
constant WS7 : std_logic_vector(3 downto 0) := "0111";
constant WS8 : std_logic_vector(3 downto 0) := "1000";

constant SYN : std_logic_vector(7 downto 0) := (X"16");

begin

--***            *--
--**  From USB  **--
--*            ***--
process( CLK, RST ) begin
	if ( RST ='1') then
		TGT_WR   <= '0';
		RD_BSY   <= '0';
		RD_STATE <= RS0;
	elsif( CLK'event and CLK='1') then
		case RD_STATE is
			when RS0 =>
				if( USB_RD='1' and IBIT8=SYN ) then
					OBIT32(31 downto 24) <= IBIT8;
					RD_STATE <= RS1;
				else
					RD_STATE <= RS0;
				end if;
			when RS1 =>
				if( USB_RD='1') then
					OBIT32(23 downto 16) <= IBIT8;
					RD_STATE <= RS2;
				else
					RD_STATE <= RS1;
				end if;
			when RS2 =>						
				if( USB_RD='1') then
					OBIT32(15 downto 8) <= IBIT8;
					RD_STATE <= RS3;
				else						
					RD_STATE <= RS2;
				end if;
			when RS3 =>
				if( USB_RD='1') then
					RD_BSY   <= '1';
					OBIT32(7 downto 0) <= IBIT8;
					RD_STATE <= RS4;
				else
					RD_STATE <= RS3;
				end if;
			when RS4 =>				
				if( TGT_RDY='1') then
					RD_BSY   <= '0';
					TGT_WR   <= '1';
					RD_STATE <= RS5;
				else
					RD_STATE <= RS4;
				end if;
			when RS5 =>
					TGT_WR   <= '0';
					RD_STATE <= RS0;
			when others => RD_STATE <= RS0;
		end case;
	end if;
end process;

--***               *--
--**  From Target  **--
--*               ***--
process( CLK, RST ) begin
	if ( RST ='1') then
		USB_WR   <= '0';
		WR_BSY   <= '0';
		WR_STATE <= WS0;
	elsif( CLK'event and CLK='1') then
		case WR_STATE is
			when WS0 =>
				if( TGT_RD='1' and IBIT32(31 downto 24)=SYN ) then
					BIT32    <= IBIT32;
					WR_BSY   <= '1';
					WR_STATE <= WS1;
				else
					WR_STATE <= WS0;
				end if;
			when WS1 =>
				if( USB_RDY='1') then
					OBIT8    <= BIT32(31 downto 24);
					USB_WR   <= '1';
					WR_STATE <= WS2;
				else
					WR_STATE <= WS1;
				end if;
			when WS2 =>
				USB_WR   <= '0';
				WR_STATE <= WS3;
			when WS3 =>
				if( USB_RDY='1') then
					OBIT8    <= BIT32(23 downto 16);
					USB_WR   <= '1';
					WR_STATE <= WS4;
				else
					WR_STATE <= WS3;
				end if;
			when WS4 =>
				USB_WR   <= '0';
				WR_STATE <= WS5;
			when WS5 =>
				if( USB_RDY='1') then
					OBIT8    <= BIT32(15 downto 8);
					USB_WR   <= '1';
					WR_STATE <= WS6;
				else
					USB_WR   <= '0';
					WR_STATE <= WS5;
				end if;
			when WS6 =>
				USB_WR   <= '0';
				WR_STATE <= WS7;
			when WS7 =>
				if( USB_RDY='1') then
					OBIT8    <= BIT32(7 downto 0);
					USB_WR   <= '1';
					WR_BSY   <= '0'; 
					WR_STATE <= WS8;
				else
					USB_WR   <= '0';
					WR_STATE <= WS7;
				end if;
			when WS8 =>
				USB_WR   <= '0';
				WR_STATE <= WS0;
			when others => WR_STATE <= WS0;
		end case;
	end if;
end process;

end RTL;

